// ModelSwarm.java


import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.activity.ActionGroupImpl;
import swarm.activity.ScheduleImpl;
import swarm.activity.Activity;

import swarm.space.Grid2dImpl;
import swarm.collections.ListImpl;

public class ModelSwarm extends SwarmImpl
{
    // Dichiaro i paramentri del modello e i loro valori standard
    public int worldXSize = 80, worldYSize = 80;
    public double consumerDensity = 0.10;
    public double enterpriceDensity = 0.10;
    public int endTime = 125;
    public int middleConsumerPrice = 50;
    public int middleEnterprisePrice = 50;
    public int start = 12345;
    public int optimism =1;
    public int consumerWalk = 3;
    public int rangeConsumerPrice = 5;
    public int rangeEnterprisePrice = 5;
    public int researchCost = 1;
// dichiaro le variabili non standard
    public MarketSpace marketSpace;
    public Grid2dImpl consumerSpace;
    public ListImpl consumerList;
    static public Grid2dImpl enterpriceSpace;
    //occorre renderla static perch vado a passarla al consumatore per la prima verifca di posizione, altrimenti va in errore le imprese una volta create non si muovo quindi l'attributo static non disturba
    public ListImpl enterpriceList;
    public ScheduleImpl modelSchedule;
   
// Questo  il construttore per il nuovo model swarm
    public ModelSwarm(Zone azone)
    {
       
	super(azone);

	// Build a customized probe map.  Without a custom probe map
	// the default is to show all variables and messages.  Here we
	// choose to customize the appearance of the probe, giving a
	// nicer interface.

	// Create the probe map and give it the ModelSwarm class.
	EmptyProbeMapImpl probeMap = 
	    new EmptyProbeMapImpl(azone, getClass());

	//inseriamo le variabili che si vuole avere all'interno del
	//men generato dall'observer.swarm
	probeMap.addProbe(getProbeForVariable("worldXSize"));
	probeMap.addProbe(getProbeForVariable("worldYSize"));
	probeMap.addProbe(getProbeForVariable("consumerDensity"));
	probeMap.addProbe(getProbeForVariable("enterpriceDensity"));
	probeMap.addProbe(getProbeForVariable("endTime"));
	probeMap.addProbe(getProbeForVariable("middleConsumerPrice"));
	probeMap.addProbe(getProbeForVariable("rangeConsumerPrice"));
	probeMap.addProbe(getProbeForVariable("middleEnterprisePrice"));
	probeMap.addProbe(getProbeForVariable("rangeEnterprisePrice"));
	probeMap.addProbe(getProbeForVariable("optimism"));
	probeMap.addProbe(getProbeForVariable("researchCost"));
	probeMap.addProbe(getProbeForVariable("consumerWalk"));
	probeMap.addProbe(getProbeForVariable("start"));
	
	// And finaly install our probe map into the probeLibrary.
	// Note that this library was created by initSwarm().

	Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());

    }

    private boolean checkEnterprice (int x, int y){
	return (enterpriceSpace.getObjectAtX$Y(x,y)!=null);

     }

 //questo  il metodo che permette il controllo della posizione delle imprese
    // che fa ritornare 
    //le coppie di x e y in cui  presente una impresa

   
    public int  getWalk (){
     

	return consumerWalk;
          }
	     
    //questo metodo restituisce il passo del consumatore, 
    //giro poi il valore allo StandardConsumer
    static public StandardEnterprice getEnterprice (int x, int y){

	return ((StandardEnterprice)enterpriceSpace.getObjectAtX$Y(x,y));
	//il metodo restituisce l'impresa nella pos X Y, 
	//conversione da obj a standardEnterprice, occorre forzare per evitare
	//messaggio di errore, questo serve per il randomWalk
	//rendendo la classe static si supera un precedente errore in cui non 
	//passava i valori

     }

    //     public List getConsumerList () {
    //return consumerList;
    // }
    //restituisce la consumerList aggiunto per il grafico


    public Object buildObjects()
    {
	int x, y, num, conta;
	//la variabile conta verifica quanti consumatori nascono su di un 
	//impresa ed incrementa il valore	
	StandardConsumer aconsumer;
	double cP; 
	StandardEnterprice aenterprice;
	double ccP;
	int cO;
	int cW;
	int cR;
	//creo tutte le variabili neccessarie al consumatore	
	// use the parent class buildObject() method to initialize the
	// process
	super.buildObjects();
		Globals.env.randomGenerator.setStateFromSeed(start);
		//questo metodo aggiorna il punto di partenza facendo lpartire la simulazione 
		//ogni volta da un punto diverso definito dalla variabile start che  settabile
		// a video dal men del Model Swarm
	// Ora creiamo gli oggetti del model.
	// First create the marketspace and seed it with food.
	marketSpace = new MarketSpace(Globals.env.globalZone, 
				  worldXSize, worldYSize);
	

	// Then create a 2-D grid that will be used to keep track of
	// each bug's position, insuring that no two bugs will ever be
	// on the same cell. Initialize the grid to be empty.
	consumerSpace = new Grid2dImpl(Globals.env.globalZone, 
				   worldXSize, worldYSize);
	consumerSpace.fastFillWithObject(null);
	//griglia in cui spargo i consumatori
	enterpriceSpace = new Grid2dImpl(Globals.env.globalZone, 
				   worldXSize, worldYSize);
	//griglia in  cui spargo le imprese
	enterpriceSpace.fastFillWithObject(null);
	//la griglia parte vuota (null)
	// Now create a List object to manage all the bugs we are
	// about to create.
	consumerList = new ListImpl(Globals.env.globalZone);

	enterpriceList = new ListImpl(Globals.env.globalZone);

	//creo due liste una per i consumatori ed una per le imprese
	//per gestire in memoria le caratteristiche dei consumatori
	
	//Inizio a mettere le imprese sul mercato 
	num = 0;
       	ccP = 0;
	for (y = 0; y < worldYSize; y++)
	    for (x = 0; x < worldXSize; x++)
		if ( Globals.env.uniformDblRand.getDoubleWithMin$withMax(
						  0.0, 1.0) <= enterpriceDensity)
		    {
			ccP = middleEnterprisePrice +  Globals.env.uniformIntRand.getIntegerWithMin$withMax(-rangeEnterprisePrice, +rangeEnterprisePrice);
			aenterprice = new StandardEnterprice(Globals.env.globalZone, enterpriceSpace, x, y, ++num, ccP);
		    enterpriceSpace.putObject$atX$Y(aenterprice, x, y);
		    enterpriceList.addLast(aenterprice);
		    System.out.println("L'impresa numero " + enterpriceList.getCount() + "  nella posizione " + x +  ", " + y + " con un prezzo di " + ccP); 
		    //inserisco il printline per verificare l'effettiva
		    // collocazione delle imprese nello spazio
		    //le imprese son sparse in modo casuale con una probabilit
		    //definita da enterprice density
		    }

	// Riporta il numero di imprese create sul mercato
	System.out.println(enterpriceList.getCount() + " Imprese sul mercato");


	//inserisco i consumatori sul mercato
	cW =  consumerWalk;
        cO = optimism;
        cR = researchCost;
	cP = 0;
	conta=0;
	num = 0;
	for (y = 0; y < worldYSize; y++)
	    for (x = 0; x < worldXSize; x++)
		if ( Globals.env.uniformDblRand.getDoubleWithMin$withMax(
						  0.0, 1.0) <= consumerDensity)
		    
		    {
			
	cP = middleConsumerPrice +  Globals.env.uniformIntRand.getIntegerWithMin$withMax(-rangeConsumerPrice, +rangeConsumerPrice);
	//inserisco il prezzo variabile dei consumatori
	//I consumatori partono con un prezzo medio defito da middleConsumerPrice a cui si aggiunge
	//+ o - 10 devo verificare se posso inserire due variabili per meglio gestirlo.
			aconsumer = new StandardConsumer(Globals.env.globalZone,  
					 consumerSpace, x, y, ++num, cP, cW, cO, cR );
		    consumerSpace.putObject$atX$Y(aconsumer, x, y);
		    consumerList.addLast(aconsumer);
		   System.out.println("Il consumatore numero " + consumerList.getCount() + " parte dalla posizione " + x +  ", " + y + " con prezzo " + cP); 
		   
 //inserisco il printline per verificare l'effettiva collocazione di partenza dei  consumatori nello spazio

		   if (checkEnterprice(x,y)) {System.out.println( "Il consumatore n " + consumerList.getCount()+ " in " + x + " , " + y + "  su una impresa");
		   conta++;  }
		    }
	//Qui verifico se il consumatore parte dalla medesima posizione in cui
	//vi  una impresa. Il controllo avviene grazie al metodo definito sopra
	// Riporta il numero di consumatori creati sul mercato
	System.out.println(consumerList.getCount() + " consumatori sul mercato, di cui su posizione di imprese " + conta);

	
	
	

	return this;



    }
    
    
    // The next three methods return some useful information about the
    // created ModelSwarm to the caller.  They will be used by the
    // Observer Swarm.
    public ListImpl getConsumerList()
    {
	return consumerList;
    }
  public ListImpl getEnterpriceList()
    {
	return enterpriceList;
    }

    public Grid2dImpl getWorld()
    {
	return consumerSpace;
    }

    public MarketSpace getMarket()
    {
	return marketSpace;
    }

    // This is the method a) for building the list of actions for
    // these objects to accomplish and b) for scheduling these actions
    // in simulated time.
    public Object buildActions()
    {
	Selector sel;
	ActionGroupImpl modelActions;

	// First, use the parent class to initialize the process.
	super.buildActions();

	// Then create an ActionGroup object and insert both a
	// randomWalk message to every bug in the bug list and a
	// checkTime message to modelSwarm. 
	modelActions = new ActionGroupImpl(getZone());

	sel = SwarmUtils.getSelector("StandardConsumer", "randomWalk");
	modelActions.createActionForEach$message(consumerList, sel);

	sel = SwarmUtils.getSelector(this, "checkTime");
	modelActions.createActionTo$message(this, sel);

	// Now create the schedule and set the repeat interval to unity.
	modelSchedule = new ScheduleImpl(getZone(), 1);

	// Finally, insert the action list into the schedule at period zero
	modelSchedule.at$createAction(0, modelActions);

	return this;
    }

    // This method specifies the context in which the model is to be run.
    public Activity activateIn(Swarm swarmContext)
    {
	// Use the parent class to activate ourselves in the context
	// passed to us.
	super.activateIn(swarmContext);

	// Then activate the schedule in ourselves.
	modelSchedule.activateIn(this);

	// Finally, return the activity we have built.
	return getActivity();
    }
	
    // This is a pretty crude method to end the simulation after an
    // arbitrary number of periods given by the endTime parameter. If
    // the simulation time returned by getCurrentTime() is greater
    // than endTime, we terminate the modelSwarm activity.  The
    // ObserverSwarm will pick this up and handle it.  Note that this
    // is no longer strictly necessary since the user can now stop the
    // simulation at any time using the control panel.  Still, this is
    // a placeholder for a more sophisticated end-of-simulation
    // routine to be introduced later.
    public void checkTime()
    {
	if (Globals.env.getCurrentTime() >= endTime)
	    getActivity().terminate();

        return;
    }
}
  
